# Script to convert table 1 of USGS Open-File Report 02-322 into a shapefile.
# The attributes are here given labels (in the report they do not have labels).
#
# Uses shpcreate, shpadd, dbfcreate, and dbfadd utilities from Frank Warmerdam's
# Shapelib (developed with version 1.2.9)
#
# So this script reads the text of the table (excised from the word document
# that is on the web) and outputs a shell script that, with the shapelib
# utilities in the execution path, will create the shapefile.
#
# Peter N. Schweitzer (U.S. Geological Survey, Reston VA 20192)

# Column headings from table1.doc
# GRAVITY STATION NAME
# LATITUDE DEG MIN
# LONGITUDE DEG MIN
# ELEV (FT)
# OBSERVED GRAVITY (MGAL)
# ACC CODES
# FAA (MGAL)
# SBA (MGAL)
# TERRAIN CORRECTION INNER OUTER (MGAL)
# CBA (MGAL)
# ISOSTATIC ANOMALY (MGAL)
# DATA SET CODES
# SECOND ELEVATION ANOMALY
# ISOSTATIC CODE

# From the documentation, the data dictionary:
# 1-4        a4    Station name
# 5-8        a4    Auxiliary station name, descriptor, or addition to station name
# 9          1x    Blank
# 10-11      f2.0  Latitude of station in degrees
# 12         1x    Blank
# 13-17      f5.2  Minutes of latitude to 0.01 minute
# 18         1x    Blank
# 19-21      f3.0  Longitude in degrees
# 22         1x    Blank
# 23-27      f5.2  Minutes of longitude to 0.01 minute
# 28         1x    Blank
# 29-35      f7.1  Elevation in feet to 0.1 feet
# 36         1x    Blank
# 37-45      f9.2  Observed gravity to 0.01 milligal
# 46         1x    Blank
# 47         a1    Location source code
# 48         a1    Gravity source code
# 49         a1    Elevation source code
# 50         a1    Simple Bouguer anomaly accuracy code
# 51         1x    Blank
# 52-58      f7.2  Free air anomaly to 0.01 milligal
# 59         1x    Blank
# 60-66      f7.2  Simple Bouguer anomaly to 0.01 milligal
# 67         1x    Blank
# 68-73      f6.2  Inner-zone terrain correction from station to 0.39 km to 0.01 milligal
# 74         1x    Blank
# 75-80      f6.2  Terrain correction from 0.39 km to 166.7 km to 0.01 milligal
# 81-83      3x    Blank
# 84-90      f7.2  Complete Bouguer anomaly to 0.01 milligal
# 91         1x    Blank
# 92-98      f7.2  Isostatic anomaly to 0.01 milligal
# 99         1x    Blank
# 100-103    a4    Data set code
# 104        1x    Blank
# 105        a1    Datum code
# 106        1x    Blank
# 107-112    f6.0  Second elevation in whole feet
# 113        1x    Blank
# 114        a1    Second elevation source code
# 115        1x    Blank
# 116-121    f6.1  Simple Bouguer anomaly based on second elevation to 0.1 milligal
# 122        1x    Blank
# 123-126    a4    Isostatic anomaly code

set dbf_file "facts.dbf"
set shp_file "facts.shp"

puts "dbfcreate $dbf_file -s stname 4 -s auxname 4 -n latdeg 2 0 -n latmin 5 2 -n londeg 3 0 -n lonmin 5 2 -n elevft 7 1 -n obsgrav 9 2 -s locsrc 1 -s gravsrc 1 -s elevsrc 1 -s sbaacc 1 -n faa 7 2 -n sba 7 2 -n iztc 6 2 -n tc 6 2 -n cba 7 2 -n ia 7 2 -s dataset 4 -s datum 1 -n elev2ft 6 0 -s elev2src 1 -n sba2elev 6 1 -s iacode 4"
puts "shpcreate $shp_file point"

set in [open table1.txt r]
while {[eof $in] == 0} {
	set line [gets $in]
	if {[string length $line] > 0} {
		set line " $line"
		set stname   [string trim [string range $line   1   4]]
		set auxname  [string trim [string range $line   5   8]]
		set latdeg   [string trim [string range $line  10  11]]
		set latmin   [string trim [string range $line  13  17]]
		set londeg   [string trim [string range $line  19  21]]
		set lonmin   [string trim [string range $line  23  27]]
		set elevft   [string trim [string range $line  29  35]]
		set obsgrav  [string trim [string range $line  37  45]]
		set locsrc   [string trim [string range $line  47  47]]
		set gravsrc  [string trim [string range $line  48  48]]
		set elevsrc  [string trim [string range $line  49  49]]
		set sbaacc   [string trim [string range $line  50  50]]
		set faa      [string trim [string range $line  52  58]]
		set sba      [string trim [string range $line  60  66]]
		set iztc     [string trim [string range $line  68  73]]
		set tc       [string trim [string range $line  75  80]]
		set cba      [string trim [string range $line  84  90]]
		set ia       [string trim [string range $line  92  98]]
		set dataset  [string trim [string range $line 100 103]]
		set datum    [string trim [string range $line 105 105]]
		set elev2ft  [string trim [string range $line 107 112]]
		set elev2src [string trim [string range $line 114 114]]
		set sba2elev [string trim [string range $line 116 121]]
		set iacode   [string trim [string range $line 123 126]]
		puts "dbfadd $dbf_file \"$stname\" \"$auxname\" \"$latdeg\" \"$latmin\" \"$londeg\" \"$lonmin\" \"$elevft\" \"$obsgrav\" \"$locsrc\" \"$gravsrc\" \"$elevsrc\" \"$sbaacc\" \"$faa\" \"$sba\" \"$iztc\" \"$tc\" \"$cba\" \"$ia\" \"$dataset\" \"$datum\" \"$elev2ft\" \"$elev2src\" \"$sba2elev\" \"$iacode\""
		set x [expr $londeg + $lonmin/60.0]
		set x [expr -$x]
		set y [expr $latdeg + $latmin/60.0]
		puts "shpadd $shp_file $x $y"
		}
	}
close $in

# From table2.doc
#
# 1-8    a8   DATA SET CODE
# 9      1x   Blank
# 10-19  a10  PROJECT NAME
# 20     1x   Blank
# 21-35  a15  TRAVERSE NAME
# 36     1x   Blank
# 37-46  a10  DATA SET DATE
# 47     1x   Blank
# 48-51  a4   METER
# 52-53  2x   Blank
# 54     a1   METER TYPE
# 55     1x   Blank
# 56-62  f7.5 METER FACTOR
# 63     1x   Blank
# 64-65  i2   GMT
# 66-68  3x   Blank
# 69-75  a7   PROJECT CHIEF
# 76     1x   Blank
# 77-90  a14  OBSERVERS

set dbf_file "dataset.dbf"

set in [open table2.txt r]

puts "dbfcreate $dbf_file -s dscode 8 -s project 10 -s traverse 15 -s dsdate 10 -s meter 4 -s metertyp 1 -n factor 7 5 -s gmt 2 -s chief 7 -s obsrvrs 14"

while {[eof $in] == 0} {
	set line [gets $in]
	if {[string length $line] > 0} {
		set line " $line"
		set dscode   [string trim [string range $line  1  8]]
		set project  [string trim [string range $line 10 19]]
		set traverse [string trim [string range $line 21 35]]
		set dsdate   [string trim [string range $line 37 46]]
		set meter    [string trim [string range $line 48 51]]
		set metertyp [string trim [string range $line 54 54]]
		set factor   [string trim [string range $line 56 62]]
		set gmt      [string trim [string range $line 64 65]]
		set chief    [string trim [string range $line 69 75]]
		set obsrvrs  [string trim [string range $line 77 90]]

		puts "dbfadd $dbf_file \"$dscode\" \"$project\" \"$traverse\" \"$dsdate\" \"$meter\" \"$metertyp\" \"$factor\" \"$gmt\" \"$chief\" \"$obsrvrs\""
		}
	}
close $in
